<?php
/**
 * LICENSE
 *
 * This software is governed by the CeCILL-C  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 *
 * @author Copyright (c) PadCMS (http://www.padcms.net)
 * @version $DOXY_VERSION
 */

class HandlerLocatorTest extends PHPUnit_Framework_TestCase
{
    protected $oLocator = null;

    public function setUp()
    {
        //GIVEN
        $this->oLocator = AM_Handler_Locator::getInstance();
    }

    public function testShouldSetHandlerClassName()
    {
        //WHEN
        $this->oLocator->setHandler('mock', 'Mock_Handler');

        //THEN
        $this->assertTrue($this->oLocator->getHandler('mock') instanceof Mock_Handler);
    }

    public function testShouldSetHandlerObject()
    {
        //GIVEN
        $handler = new Mock_Handler();
        $handler->sProperty = 'value';

        //WHEN
        $this->oLocator->setHandler('mock', $handler);

        //THEN
        $this->assertTrue($this->oLocator->getHandler('mock') instanceof Mock_Handler);
        $this->assertEquals('value', $this->oLocator->getHandler('mock')->sProperty);
    }

    public function testShouldThrowExceptionWhenSetIncorrectHandlerClass()
    {
        //GIVEN
        $this->setExpectedException('AM_Handler_Locator_Exception', '', 502);

        //WHEN
        $this->oLocator->setHandler('mock', 'Mock_Wrong_Handler');
    }

    public function testShouldThrowExceptionWhenSetIncorrectHandlerObject()
    {
        //GIVEN
        $oHandler = new Mock_Wrong_Handler();
        $this->setExpectedException('AM_Handler_Locator_Exception', '', 502);

        //WHEN
        $this->oLocator->setHandler('mock', 'Mock_Wrong_Handler');
    }

    public function testShouldThrowExceptionWhenTryingToGetUnsetedHandler()
    {
        //GIVEN
        $this->setExpectedException('AM_Handler_Locator_Exception', '', 501);

        //WHEN
        $this->oLocator->getHandler('false');
    }
}

class Mock_Handler extends AM_Handler_Abstract
{
    public $sProperty = null;
}

class Mock_Wrong_Handler
{}